package org.jabricks.widgets.treetable;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.tree.TreePath;

// import org.jabricks.widgets.examples.TreeTableData;
import org.jabricks.widgets.renderers.FloatRenderer;
import org.jabricks.widgets.treetable.ITreeTableListener;
import org.jabricks.widgets.treetable.JTreeTable;
import org.jabricks.widgets.treetable.ObjectModel;
import org.jabricks.widgets.treetable.ObjectNode;
import org.jabricks.widgets.treetable.TreeTableModel;

public class ExampleTreeTable implements ITreeTableListener
{
    private   JFrame        frame    ;
    private   ObjectModel   modelData;  // Модель данных    
    private   JTreeTable    treeTable;  // Компонент JTreeTable    
    
    // флаг определения языка : true русс., false англ. 
    private   boolean  lang_ru       = true;      
    // заголовки таблицы, английский язык
	private String[]   titles_en     = { "Name"        , "Quantity"  , "Cost"     };
    // заголовки таблицы, русский язык 
	private String[]   titles_ru     = { "Наименование", "Количество", "Стоимость"};
    // наименование колонок таблицы
	private String[]   column_names  = { "name"        , "qty"       , "cost"     };
    // типы значений по колонкам таблицы
	private Class<?>[] column_types  = { TreeTableModel.class, Integer.class, Float.class};
    //размеры колонок таблицы
	private int        cols_width[]  = {350, 110, 120};
	
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    /**
     * Конструктор класса
     */
    public ExampleTreeTable()
    {
    	try {
			UIManager.setLookAndFeel(UIManager.getLookAndFeel());
		} catch (UnsupportedLookAndFeelException e1) {}  

    	JFrame.setDefaultLookAndFeelDecorated(true);
//		JTreeTable.setLookAndFeel(3);

		frame = new JFrame("Пример использования TreeTable");
		frame.setSize(new Dimension(600, 400));

		frame.addWindowListener(new WindowAdapter() {
		    public void windowClosing(WindowEvent we) {
		    	System.exit(0);
		    }
		});

		// Объект данных
		TreeTableData data = new TreeTableData();

		// Корневая запись
		Record record = data.getRootRecord();

		ObjectModel.setSignificantFields("name", "leaf");

		// Интерфейсный корневой объект иерархической структуры данных
		ObjectNode rootNode = ObjectModel.convertRecord2Node (record);

		// Модель представления данных в таблице TreeTable  
		modelData = new ObjectModel(column_names, titles_ru, column_types);
		
//		modelData.setRootNode(rootNode);
		
		// Создание компонента
		treeTable = new JTreeTable(modelData);
		treeTable.drawTableHeaderRaised();
		treeTable.setName("TreeTableName");

		// Подключение метода callback
		treeTable.setParentCallBack(this);

		// Настройка компонента
		treeTable.setColumnsWidth(cols_width);
		treeTable.setRowHeight(22);
	    // Подключение FloatRenderer библиотеки gui_widgets
		treeTable.setDefaultRenderer(Float.class, new FloatRenderer());
		treeTable.setAutoResizeColumn (JTable.AUTO_RESIZE_OFF); 
//		treeTable.setAutoResizeColumn (JTable.AUTO_RESIZE_ALL_COLUMNS); 
		
		// Панель кнопок управления
		JPanel pnlButtons = new JPanel();
		pnlButtons.setPreferredSize(new Dimension(400, 40));
		pnlButtons.setSize         (new Dimension(400, 40));
		pnlButtons.setBackground(Color.LIGHT_GRAY);
		
		createButtonsPanel(pnlButtons);
		frame.getContentPane().add(pnlButtons, BorderLayout.SOUTH );
		frame.getContentPane().add(new JScrollPane(treeTable), BorderLayout.CENTER);
//		frame.pack();

	    // Загрузка данных 
		modelData.setRootNode(rootNode);
		treeTable.updateUI();
		
		//~~~ Выделение записи в таблице ~~~
	    // путь выделяемой записи в дереве
		final int[] rows = {0,1,2,4};
	    // последний узел в таблице выделить
		treeTable.selectRow(rows, true);
//		treeTable.selectRow(0, true);  

		frame.setVisible(true);
    }
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    /**
     * Метод вывода сообщения о выделенной записи
     */
	private void onClickNode(Record record)
	{
		if (record != null) {
			String text = "id=" + record.getId() + ", name=" + record.getName();
			JOptionPane.showMessageDialog(null,   // TreeTableExample.this
					   "Вы выбрали запись : \n" + text, 
					   "Выделенная запись", JOptionPane.INFORMATION_MESSAGE);
		}
	}
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    /**
     * Функция обратного вызова.
     * Компонент TreeTable вызывает данную функцию и передает
     * ей в качестве параметра выделенную запись типа Object
     */ 
	@Override
    public void callBack(Object obj)
    {
		try {
			if (obj != null) {
	            // Определение выделенной записи
				ObjectNode node = (ObjectNode) obj;
				
	            // Определение флага языка локализации
				lang_ru = !lang_ru;
	            // Определение заголовков колонок таблицы
				if (lang_ru)
					treeTable.setColumnTitles(titles_ru);
				else
					treeTable.setColumnTitles(titles_en);
				// Получение объекта записи выделенного узла
				onClickNode((Record) node.getRecord());
			}
		} catch (Exception e) {}
    }
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    /**
     * Создание панели кнопок управления.
     * В методе определяются кнопки управления, к слушателям
     * которых подключаются методы выполнения транзакций :
     * recordAdd(), recordUpdate(), recordDel()
     */
    private void createButtonsPanel(JPanel pnlButtons)
    {
        JButton btnAdd    = new JButton("Добавить" );
        JButton btnSave   = new JButton("Сохранить");
        JButton btnDelete = new JButton("Удалить"  );

        btnAdd.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				recordAdd();
			}        	
        });
        btnSave.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				recordUpdate();
			}        	
        });
        btnDelete.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				recordDel();
			}        	
        });
        pnlButtons.add (btnAdd   );
        pnlButtons.add (btnSave  );
        pnlButtons.add (btnDelete);        
    }
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    /**
     * Метод определения выделенной записи.
     * Используется при выполнении транзакций
     */
    private Object getTreeTableSelectedObject()
    {
		Object object = null;
    	TreePath path = treeTable.getTree().getSelectionPath();
    	if (path != null)
    		object = (ObjectNode) path.getLastPathComponent();
    	return object;
    }
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    /**
     * Метод добавления новой записи в таблицу : 
     *  - создание новой записи типа Record,
     *  - создание интерфейсного объекта типа ObjectNode
     *  - включение Record и в объект ObjectNode
     *  - добавление интерфейсного объекта в таблицу
     *
     */
    private void recordAdd()
    {
        // Путь выделенного объекта
    	TreePath    path   = treeTable.getTree().getSelectionPath();
        // Родительский объект в таблице
    	ObjectNode  parent = (ObjectNode)getTreeTableSelectedObject();

        // Выделенная запись папка (узловая запись) или лист?
    	if ((parent != null) && parent.isLeaf()) {
            // Переопределение родительской записи
    		parent = parent.getParent();
    		path   = path.getParentPath();
    		treeTable.getTree().getSelectionModel().setSelectionPath(path);
    	}
        // Родительская запись
    	Record record = null;
    	if (parent != null)
    		record = (Record)parent.getRecord();
    	if (record != null) {
            // Новая запись
    		Record  item = TreeTableData.createNewRecord();
            // Новый интерфейсный объект
    		ObjectNode object = new ObjectNode(item);
            // Определение родителя 
    		object.setParent((ObjectNode)parent);
            /*
             * Добавление новой записи в таблицу. Последний 
             * логический параметр метода addNode определяет
             * необходимость выделения новой добавленной записи
             */
    		treeTable.addNode(parent, object, path, modelData, true);
    	}
    }
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    /**
     * Метод обновления записи. В методе случайным образом 
     * определяются значения полей qty, cost выделенной записи.
     */
    private void recordUpdate()
    {
        // Выделенный объект таблицы
    	ObjectNode    object = (ObjectNode)getTreeTableSelectedObject();
        // Запись выделенного объекта
    	Record  record = null;

    	if (object != null)
    		record = (Record)object.getRecord();
    	if (record != null) {
            // Обновление значений выделенной записи
    		record.setQty((int)(Math.random() * 100));
	    	record.setCost((float) (record.getQty() * Math.random() * 100));
	    	
	        // Путь выделенной записи
	    	TreePath path = treeTable.getTree().getSelectionPath();
	    	// Обновление выделенной записи в таблице
	    	treeTable.updateNode(object, path, modelData, true);
    	}
    }
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    /**
     * Метод удаления записи
     */
    private void recordDel()
    {
        // Выделенная в таблице запись
    	ObjectNode object = (ObjectNode)getTreeTableSelectedObject();
        // Удаление записи
		treeTable.deleteNode(object, treeTable.getTree().getSelectionPath(), modelData);
    }
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    public static void main(String[] args)
    {
	    new ExampleTreeTable();
    }
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
}
