package org.jabricks.widgets.treetable;

import java.util.ArrayList;
import java.util.List;

public class TreeTableData
{
	private Object[][]   basket  = {{ 0, 0, "Список товаров", null, null}, 
		                            { 1, 0, "Продукты"      , null, null},
		                            { 3, 1, "Молочные"      , null, null}, 
		                            { 4, 1, "Хлеб"          , null, null}, 
		                            { 5, 0, "Промтовары"    , null, null}, 
		                            { 6, 4, "Хлеб дарницкий",    1,  40f},
		                            { 7, 4, "Батон нарезной",    1,  48f},
		                            { 8, 3, "Сметана"       ,    1,  64f},
		                            { 9, 3, "Молоко"        ,    3, 198f},
		                            {10, 5, "Лампочки"      ,    2, 180f},
		                            {11, 5, "Батарейки"     ,    4,  80f},
		                            {12, 5, "Замок"         ,    1,1200f},
		                            {13, 5, "Кран Kaiser"   ,    1,2400f}};
	private Record       root    = null;
    static List<Record>  records = null;
    //---------------------------------------------------------------------------------------------
	public TreeTableData()
	{
		createObjectRoot();
	}
    //---------------------------------------------------------------------------------------------
	public Record getRootRecord()
	{
		return root;
	}
    //---------------------------------------------------------------------------------------------
	/**
	 * Метод преобразования массива basket в список records и
	 * вызов метода формирования иерархической структуры данных.
	 */
	private void createObjectRoot()
	{
	    records = new ArrayList<Record>();
		for (int i = 0; i < basket.length; i++) {
			Record or = new Record();
			or.setId   (Integer.valueOf((Integer) basket[i][0]));
			or.setPid  (Integer.valueOf((Integer) basket[i][1]));
			or.setName ((String)                  basket[i][2]);
			if (basket[i][3] != null)
				or.setQty  (Integer.valueOf((Integer) basket[i][3]));
			if (basket[i][4] != null)
				or.setCost (Float.valueOf((Float) basket[i][4]));
			or.setLeaf((basket[i][3] != null) && (basket[i][4] != null));
			records.add(or);
		}
		createRootRecord(records);
	}
    //---------------------------------------------------------------------------------------------
	/**
	 * Метод определения корневой записи.
	 * @param records список записей
	 */
	private void createRootRecord(List<Record> records)
	{
		root = records.get(0);
		root.setLeaf(false);

		createTreeRecord(root, records);
	}
    //---------------------------------------------------------------------------------------------
	/**
	 * Рекурсивная процедура построения дерева записей
	 * @param record текущий узел
	 * @param records список записей
	 */
	private void createTreeRecord(Record record, List<Record> records)
	{
		for (int i = 1; i < records.size(); i++) {
			if (records.get(i).getPid() == record.getId()) {
				Record child = records.get(i);
				record.getChildren().add(child);
				for (int j = 1; j < records.size(); j++) {
					if (records.get(j).getPid() == child.getId()) {
						child.getChildren().add(records.get(j));
						if (!records.get(j).isLeaf()) {
							createTreeRecord(records.get(j), records);
						}
					}
				}
			}
		}
	}
    //---------------------------------------------------------------------------------------------
	/**
	 * Метод определения идентификатора новой записи
	 * @return id новой записи
	 */
	public static int getMaxID()
	{
		int id = 0;
		for (int i = 0; i < records.size(); i++) {
			if (records.get(i).getId() > id)
				id = records.get(i).getId().intValue();
		}
		return id + 1;
	}
    //---------------------------------------------------------------------------------------------
	/**
	 * Метод формирования новой записи
	 * @return новая запись типа Record
	 */
	public static Record createNewRecord()
	{
		Record record = new Record();
		record.setId(getMaxID());
		record.setQty ((int)(Math.random() * 10));
		float f = (float) ((float) record.getQty() * Math.random() * 100);
		f = ((int)f * 10) / 10;
		record.setCost(f);
		record.setName("Добавленная запись-" + record.getId());
		
		records.add(record);
		return record;
	}
    //---------------------------------------------------------------------------------------------
}
